#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

#include <Inventor/SoDB.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/sensors/SoTimerSensor.h>

void TimerCallback(void *, SoSensor *);

/* These are defined as external since there are two functions which 
	must both access these variables */
SoTransform 	*cube;
SoTransform 	*cylinder;

int main(int argc, char **argv) {
	Widget				window;
	SoSeparator 			*root;
	SoInput 			*SceneInput;
	SoXtExaminerViewer 		*viewer;
	SoTimerSensor 			*TimerSensor;

	/* Initialize Inventor and Xt; create new viewer */
        window = SoXt::init(*argv);
        viewer = new SoXtExaminerViewer(window);

	/* Make the new stuff */
	root 				= new SoSeparator;
	SceneInput 			= new SoInput;

	/* Parse the inventor file */
	SceneInput->openFile("gview.iv");
			root = SoDB::readAll(SceneInput);
	SceneInput->closeFile();

	/* Get stuff from the files */
	cube 		= (SoTransform *) SoNode::getByName("cube_transform");
	cylinder 	= (SoTransform *) SoNode::getByName("cylinder_transform");

	/* Display SceneGraph in viewer */
	viewer->setSceneGraph(root);
	viewer->show();

	/* Schedule invocation of a function with void* arguments */
	TimerSensor = new SoTimerSensor(TimerCallback, NULL);
        TimerSensor->schedule();

	/* Display viewer window and initialize event handler */
	SoXt::show(window);
	SoXt::mainLoop();
}

void TimerCallback(void *arguments, SoSensor *sensor) {
        float cube_translation, cylinder_translation;

	/* Modify scene according to user input */
	printf("Type a displacement for cube and then cylinder\n");
	scanf("%f%f", &cube_translation, &cylinder_translation);
	cube->translation.setValue(cube_translation, 0, 0);	
	cylinder->translation.setValue(cylinder_translation, 0, 0);	

	/* Schedule another invocation of the function */
        sensor->schedule();
}
