#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/sensors/SoTimerSensor.h>

SoSeparator *makeArch(void);

static float vertices[60][3] = {	/* Array of 60 3-dimensional vertex coordinates */
	// 1st row
	{-13.0,  0.0, 1.5}, {-10.3, 13.7, 1.2}, { -7.6, 21.7, 1.0},
	{ -5.0, 26.1, 0.8}, { -2.3, 28.2, 0.6}, { -0.3, 28.8, 0.5},
	{  0.3, 28.8, 0.5}, {  2.3, 28.2, 0.6}, {  5.0, 26.1, 0.8}, 
	{  7.6, 21.7, 1.0}, { 10.3, 13.7, 1.2}, { 13.0,  0.0, 1.5},

	// 2nd row
	{-10.0,  0.0, 1.5}, { -7.9, 13.2, 1.2}, { -5.8, 20.8, 1.0},
	{ -3.8, 25.0, 0.8}, { -1.7, 27.1, 0.6}, { -0.2, 27.6, 0.5},
	{  0.2, 27.6, 0.5}, {  1.7, 27.1, 0.6}, {  3.8, 25.0, 0.8}, 
	{  5.8, 20.8, 1.0}, {  7.9, 13.2, 1.2}, { 10.0,  0.0, 1.5},

	// 3rd row
	{-10.0,  0.0,-1.5}, { -7.9, 13.2,-1.2}, { -5.8, 20.8,-1.0},
	{ -3.8, 25.0,-0.8}, { -1.7, 27.1,-0.6}, { -0.2, 27.6,-0.5},
	{  0.2, 27.6,-0.5}, {  1.7, 27.1,-0.6}, {  3.8, 25.0,-0.8}, 
	{  5.8, 20.8,-1.0}, {  7.9, 13.2,-1.2}, { 10.0,  0.0,-1.5},

	// 4th row 
	{-13.0,  0.0,-1.5}, {-10.3, 13.7,-1.2}, { -7.6, 21.7,-1.0},
	{ -5.0, 26.1,-0.8}, { -2.3, 28.2,-0.6}, { -0.3, 28.8,-0.5},
	{  0.3, 28.8,-0.5}, {  2.3, 28.2,-0.6}, {  5.0, 26.1,-0.8}, 
	{  7.6, 21.7,-1.0}, { 10.3, 13.7,-1.2}, { 13.0,  0.0,-1.5},

	// 5th row
	{-13.0,  0.0, 1.5}, {-10.3, 13.7, 1.2}, { -7.6, 21.7, 1.0},
	{ -5.0, 26.1, 0.8}, { -2.3, 28.2, 0.6}, { -0.3, 28.8, 0.5},
	{  0.3, 28.8, 0.5}, {  2.3, 28.2, 0.6}, {  5.0, 26.1, 0.8}, 
	{  7.6, 21.7, 1.0}, { 10.3, 13.7, 1.2}, { 13.0,  0.0, 1.5}
};

int main(int argc, char **argv) {
	Widget window;
	SoSeparator *root;
	SoSeparator *arch;
	SoXtExaminerViewer *viewer;

	/* Initialize Inventor and Xt; create new viewer */
	window = SoXt::init(*argv);
	viewer = new SoXtExaminerViewer(window);

	/* Generate the Arch */
	root = new SoSeparator;
	if (!(arch = makeArch())) {
		printf("Error: Unable to generate arch\n");
		exit(1);
	}
	root->addChild(arch);

	/* Display SceneGraph in viewer */
	viewer->setSceneGraph(root);
	viewer->show();

	/* Display viewer window and initialize event handler */
	SoXt::show(window);
	SoXt::mainLoop();
}

SoSeparator *makeArch() {	// Arch Generation Function
	SoMaterial *color;
	SoSeparator *result;
	SoQuadMesh *QuadMesh;
	SoCoordinate3 *coordinates;

	result = new SoSeparator;
	result->ref();

	/* Set the color of the Arch */
	color = new SoMaterial;
	color->diffuseColor.setValue(.78, .57, .11);
	result->addChild(color);

	/* Map the verticies array to 3-dimensional coordinates */
	coordinates = new SoCoordinate3;
	coordinates->point.setValues(0, 60, vertices);
	result->addChild(coordinates);

	/* Create the Arch using the above coordinates */
	QuadMesh = new SoQuadMesh;
	QuadMesh->verticesPerRow = 12;
	QuadMesh->verticesPerColumn = 5;
	result->addChild(QuadMesh);

	result->unrefNoDelete();
	return result;
}
