#include <stdio.h>
#include <stdlib.h>

#include <Inventor/SoDB.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/sensors/SoNodeSensor.h>
#include <Inventor/sensors/SoTimerSensor.h>

int main(int argc, char **argv) {
	Widget window;
	SoXtExaminerViewer *viewer;

	window = SoXt::init(*argv);
	viewer = new SoXtExaminerViewer(window);

	/* Geometry */
        SoCube  	*thigh; 
        SoCube 		*calf;
	SoCube 		*foot;
	SoSphere 	*headSphere;
	SoCylinder 	*bodyCylinder;

	/* Transforms */
	SoTransform 	*calfTransform;
	SoTransform 	*footTransform;
	SoTransform 	*leftTransform;
	SoTransform 	*rightTransform;
	SoTransform 	*bodyTransform;
	SoTransform 	*headTransform;

	/* Groupings */
	SoGroup 	*leg;
	SoSeparator 	*leftLeg;
	SoSeparator 	*rightLeg;
	SoSeparator 	*body;
	SoSeparator 	*head;
	SoSeparator	*root;

	/* Colors */
	SoMaterial 	*bronze;
	SoMaterial 	*silver;

        calf 		= new SoCube;
        thigh 		= new SoCube;
	calfTransform 	= new SoTransform;
	foot  		= new SoCube;
	footTransform 	= new SoTransform;
	leg 		= new SoGroup;
	leftLeg 	= new SoSeparator;  
	leftTransform 	= new SoTransform;
	rightTransform 	= new SoTransform;
	rightLeg 	= new SoSeparator;  
	bronze 		= new SoMaterial;
	bodyCylinder 	= new SoCylinder;
	bodyTransform 	= new SoTransform;
	body 		= new SoSeparator;
	headSphere 	= new SoSphere;
	headTransform 	= new SoTransform;
	silver 		= new SoMaterial;
	head 		= new SoSeparator;
	root 		= new SoSeparator; 


/* LEGS */
	/* Set displacements two ways */
	footTransform->translation.setValue(0, -2, .5);
	calfTransform->translation.setValue(0, -2.25, 0.0);
	leftTransform->translation = SbVec3f(1, -4.25, 0);
	rightTransform->translation.setValue(-1, -4.25, 0);

	/* Define the thigh */
	thigh->width = 1.2;
	thigh->height = 2.2;
	thigh->depth = 1.1;

	/* Define the Calf */
	calf->width = 1;
	calf->height = 2.2;
	calf->depth = 1;

	/* Define the foot */
	foot->width = 0.8;
	foot->height = 0.8;
	foot->depth = 2;

	/* Build the ONE leg */
	leg->addChild(thigh);
	leg->addChild(calfTransform);
	leg->addChild(calf);
	leg->addChild(footTransform);
	leg->addChild(foot);

	/* Use the ONE leg twice, interpaced with transsform */
	leftLeg->addChild(leftTransform);
	leftLeg->addChild(leg);
	rightLeg->addChild(rightTransform);
	rightLeg->addChild(leg);

/* BODY */
	bronze->ambientColor.setValue(.33, .22, .27);
	bronze->diffuseColor.setValue(.78, .57, .11);
	bronze->specularColor.setValue(.99, .94, .81);
	bronze->shininess = .28;

	/* Create Body */
	bodyCylinder->radius = 2.5;
	bodyCylinder->height = 6;

	bodyTransform->translation.setValue(0.0, 3.0, 0.0);

	/* Create Robot Body from Components */
	body->addChild(bodyTransform);
	body->addChild(bronze);
	body->addChild(bodyCylinder);
	body->addChild(leftLeg);
	body->addChild(rightLeg);

/* HEAD */
	headTransform->translation.setValue(0, 7.5, 0);
	headTransform->scaleFactor.setValue(1.5, 1.5, 1.5);

	/* Create Robot Head Material */
	silver->ambientColor.setValue(.2, .2, .2);
	silver->diffuseColor.setValue(.6, .6, .6);
	silver->specularColor.setValue(.5, .5, .5);
	silver->shininess = .5;

	head->addChild(headTransform);
	head->addChild(silver);
	head->addChild(headSphere);

	/* Create Robot from Components */
	root->ref();
	root->addChild(head);
	root->addChild(body);	      

	viewer->setSceneGraph(root);
	viewer->show();

	SoXt::show(window);
	SoXt::mainLoop();
}
