#include <stdlib.h>
#include <malloc.h>
#include <math.h>

#include <Inventor/SoDB.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/sensors/SoNodeSensor.h>

int main(int argc, char **argv) {
	Widget myWindow = SoXt::init(argv[0]);
	if (myWindow == NULL) exit(1);

	SoSeparator 			*root ;
	SoGroup 			*oxygen;
	SoGroup 			*hydrogen1; 
	SoGroup 			*hydrogen2; 
	SoGroup 			*waterMolecule; 
	SoSphere 			*sphere1; 
	SoSphere 			*sphere2; 
	SoSphere 			*sphere3; 
	SoTransform 			*hydrogenXform1;
	SoTransform 			*hydrogenXform2; 
	SoMaterial 			*redPlastic;
	SoMaterial 			*whitePlastic; 
	SoXtExaminerViewer	 	*myViewer;

	root 				= new SoSeparator;
	oxygen 				= new SoGroup;
	hydrogen1 			= new SoGroup;
	hydrogen2 			= new SoGroup;
	waterMolecule 			= new SoGroup;
	sphere1 			= new SoSphere;
	sphere2 			= new SoSphere;
	sphere3 			= new SoSphere;
	hydrogenXform1 			= new SoTransform;
	hydrogenXform2 			= new SoTransform;
	redPlastic 			= new SoMaterial;
	whitePlastic 			= new SoMaterial;
	myViewer 			= new SoXtExaminerViewer(myWindow);

	root->ref();

	/* Set the color properties for the Oxygen Atom */
	redPlastic->ambientColor.setValue(1.0, 0.0, 0.0);	
	redPlastic->diffuseColor.setValue(1.0, 0.0, 0.0); 
	redPlastic->specularColor.setValue(0.5, 0.5, 0.5);
	redPlastic->shininess = 0.5;

	/* Set the color properties for the Hydrogen Atoms */
	whitePlastic->ambientColor.setValue(1.0, 1.0, 1.0);
	whitePlastic->diffuseColor.setValue(1.0, 1.0, 1.0);
	whitePlastic->specularColor.setValue(0.5, 0.5, 0.5);
	whitePlastic->shininess = 0.5;

	/* Translate the Hydrogen Atoms */
	hydrogenXform1->scaleFactor.setValue(0.75, 0.75, 0.75);	
	hydrogenXform1->translation.setValue(0.0, -1.2, 0.0);	
	hydrogenXform2->translation.setValue(1.1852, 1.3877, 0.0);

	/* Create the Hydrogen Atoms */
	hydrogen1->addChild(hydrogenXform1);
	hydrogen1->addChild(whitePlastic);
	hydrogen1->addChild(sphere2);
	hydrogen2->addChild(hydrogenXform2);
	hydrogen2->addChild(sphere3);

	/* Create the Oxygen Atom */
	oxygen->addChild(redPlastic);
	oxygen->addChild(sphere1);

	/* Create the Water Molecule Hierarchy */
	waterMolecule->addChild(oxygen); 
	waterMolecule->addChild(hydrogen1);	 
	waterMolecule->addChild(hydrogen2);

	myViewer->setSceneGraph(waterMolecule);
	myViewer->setTitle("Water Molecule");
	myViewer->show();

	SoXt::show(myWindow);
	SoXt::mainLoop();
}
